/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */


// package fi.hut.tcm.jacob.tcpip.demo;
import fi.hut.tcm.jacob.tcpip.*;
import java.awt.*;
import java.awt.event.*;
import java.net.*;

public class BallGame extends Frame {
  public static final int SERVER_PORT = 7887;

  InetAddress localAddress;
  InetAddress serverAddress;
  JacobDatagramSocket udpSocket;
  TextArea outBox, inBox;
  Button newButton, pauseButton, resumeButton;
  Sender sender;
  Thread receiveThread;

  public BallGame(String localHost, String serverHost) {
    try {
      localAddress = InetAddress.getByName(localHost);
      serverAddress = InetAddress.getByName(serverHost);
    } catch (UnknownHostException e) {
      System.err.println("Host not found");
      System.exit(1);
    }

    try {
      udpSocket = new JacobDatagramSocket(SERVER_PORT, localAddress);
    } catch (SocketException e) {
      System.err.println("Error opening socket");
      System.exit(1);
    }

    inBox = new TextArea("", 15, 13, TextArea.SCROLLBARS_NONE);
    outBox = new TextArea("",15, 13, TextArea.SCROLLBARS_NONE);
    inBox.setEditable(false);
    outBox.setEditable(false);

    MenuBar menuBar = new MenuBar();
    Menu fileMenu = new Menu("File");
    MenuItem menuExit = new MenuItem("Exit");
    BallGameExitListener exitListener = new BallGameExitListener();
    menuExit.addActionListener(exitListener);
    fileMenu.add(menuExit);
    menuBar.add(fileMenu);
    setMenuBar(menuBar);

    Panel westPanel = new Panel(new BorderLayout(2, 2));
    westPanel.add(new Label("incoming", Label.CENTER), BorderLayout.NORTH);
    westPanel.add(inBox, BorderLayout.CENTER);

    Panel eastPanel = new Panel(new BorderLayout(2, 2));
    eastPanel.add(new Label("outgoing", Label.CENTER), BorderLayout.NORTH);
    eastPanel.add(outBox, BorderLayout.CENTER);
  
    Panel southPanel = new Panel();
    newButton = new Button ("New game");
    southPanel.add(newButton);

    pauseButton = new Button("Pause game");
    resumeButton = new Button("Resume game");
    resumeButton.setEnabled(false);
    BallGamePauseListener pauseListener =
      new BallGamePauseListener(pauseButton, resumeButton);
    BallGameResumeListener resumeListener =
      new BallGameResumeListener(pauseButton, resumeButton);
    pauseButton.addActionListener(pauseListener);
    resumeButton.addActionListener(resumeListener);
    southPanel.add(pauseButton);
    southPanel.add(resumeButton);

    Button exitButton = new Button("Exit");
    exitButton.addActionListener(exitListener);
    southPanel.add(exitButton);

    GameArea gameArea = new GameArea();
    gameArea.setSize(300, 300);
    gameArea.setBackground(Color.white);
    sender = new Sender(outBox, udpSocket, serverAddress, SERVER_PORT);
    gameArea.addMouseMotionListener(sender);

    add(westPanel, BorderLayout.WEST);
    add(gameArea, BorderLayout.CENTER);
    add(eastPanel, BorderLayout.EAST);
    add(southPanel, BorderLayout.SOUTH);

    receiveThread = new Thread(new Receiver(inBox, gameArea, udpSocket));
    receiveThread.start();

    pack();
  }

  public boolean handleEvent(Event event) {
    if (event.id == Event.WINDOW_DESTROY) {
      System.exit(0);
    }
    return super.handleEvent(event);
  }

  
  public static void main(String[] args) {
    
    if (args.length != 2) {
      System.out.println("USAGE: java BallGame <local ip> <server ip>");
      System.exit(1);
    }
    BallGame window = new BallGame(args[0], args[1]);
    window.setLayout(new BorderLayout(5, 5));
    window.setTitle("Ball Game: a UDP Test Application");

    window.show();
  }
}
