/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */


import java.net.*;

// For Demo:
import fi.hut.tcm.jacob.tcpip.*;

public class BallGameServer {
  //  private InetAddress saddr;
  // int sport;
  private int x_pos, y_pos;
  private byte buffer[];
  private DatagramPacket packet;
  //private DatagramSocket my_socket;
  private JacobDatagramSocket my_socket;

  public BallGameServer(JacobDatagramSocket sock) {
    buffer = new byte[20];
    packet = new DatagramPacket(buffer, buffer.length);
    my_socket = sock;
  }

  public void serve() {
    for ( ; ; ) {
      try {
	my_socket.receive(packet);
      } catch (java.io.IOException io) {
	System.err.println("Error receiving UDP datagram: " + io);
	// Clean up the packet, just in case:
	packet = new DatagramPacket(buffer, buffer.length);
      }
      buffer = packet.getData();
      x_pos = (((int) buffer[0] & 0xff) << 8) | (((int) buffer[1]) & 0xff);
      y_pos = (((int) buffer[2] & 0xff) << 8) | (((int) buffer[3]) & 0xff);
      System.out.println("Received: X=" + x_pos + ", Y=" + y_pos);

      // Construct the packet to send back:
      x_pos = (int) (50 + (x_pos - 50) * 0.6);
      y_pos = (int) (50 + (y_pos - 50) * 0.6);
      buffer[0] = (byte) (x_pos >> 8);
      buffer[1] = (byte) (x_pos & 0xff);
      buffer[2] = (byte) (y_pos >> 8);
      buffer[3] = (byte) (y_pos & 0xff);

      try {
	my_socket.send(packet);
      } catch (java.io.IOException io) {
	System.err.println("Error sending UDP datagram: " + io);
      }
      System.out.println("  Sent: X=" + x_pos + ", Y=" + y_pos);
    }
  }

  public static void main(String argv[]) {
    BallGameServer the_server;

    if (argv.length != 1) {
      System.err.println("Usage: BallGameServer <local ip>");
      System.exit(1);
    }

    //DatagramSocket udpSocket;
    JacobDatagramSocket udpSocket;

    System.out.println("BallGameServer.java version 0.02");
    try {
      //udpSocket = new DatagramSocket(7887);
      try {
	udpSocket = new JacobDatagramSocket(7887, 
					    InetAddress.getByName(argv[0]));
	the_server = new BallGameServer(udpSocket);
	the_server.serve();
      } catch (UnknownHostException e) {
	System.err.println("Usage: BallGameServer <local ip>");
	System.exit(1);
      };
    } catch (SocketException e) {
      System.err.println("Error opening socket");
      System.exit(1);
    }

    System.exit(0);
  }
}
