/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpAddress (implementation)
    Package: tcpip
    File:    IpAddress.java


*/
package fi.hut.tcm.jacob.tcpip;
import java.util.StringTokenizer;

/**
    IpAddress
    
    A utility class for handling IPv4 addresses. Mainly useful
    for its conversion functions.
*/
public class IpAddress {
    private int addr;

    public IpAddress(String strAddr) {
    //+ IpAddress::IpAddress(String)
      StringTokenizer addrTok = new StringTokenizer(strAddr, ".", false);
      if (addrTok.countTokens() != 4) {
        throw new
	  IllegalArgumentException("Invalid ethernet address in argument: " + strAddr);
      }
      long laddr = 0;
      long multiply = 16777216L;  // 256 ^ 3, REMEMBER THE 'L' !
      for (int i = 0; i < 4; i++) {
        try {
          laddr += multiply * (long) (Integer.parseInt(addrTok.nextToken(), 10 ) & 0xff);
	  multiply /= 256;
        } catch (NumberFormatException e) {
        throw new
	  IllegalArgumentException("Invalid ethernet address in argument: " + strAddr);
        }
      }
      addr = (int) laddr;
    //- IpAddress::IpAddress(String)
    }

    public IpAddress(int intAddr) {
    //+ IpAddress::IpAddress(int)
      addr = intAddr;
    //- IpAddress::IpAddress(int)
    }

    public static String toString(int intAddr) {
    //+ IpAddress::toString(int)
      String elem;
      byte[] arr = IpAddress.toArray(intAddr);
      String str = new String(Integer.toString( arr[0] & 0xff ));
      for (int i = 1; i < 4; i++) {
	elem = Integer.toString( arr[i] & 0xff );
	str = str + "." + elem;
      }
      return str;
    //- IpAddress::toString(int)
    }

    public String toString() {
    //+ IpAddress::toString( )
      return toString(addr);
    //- IpAddress::toString( )
    }

    public static byte[] toArray(int intAddr) {
    //+ IpAddress::toArray(int)
      byte[] arr = new byte[4];
      long laddr = ((long) intAddr) & 4294967295L;  // 256 ^ 4 -1
      for (int i = 3; i >= 0; i--) {
	arr[i] = (byte) ((laddr % 256) & 0xff);
	laddr /= 256;
      }
      return arr;
    //- IpAddress::toArray(int)
    }

    public byte[] toArray() {
    //+ IpAddress::toArray( )
      return toArray(addr);
    //- IpAddress::toArray( )
    }

    public int toInt() {
    //+ IpAddress::toInt( )
      return addr;
    //- IpAddress::toInt( )
    }

}; //IpAddress

