/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  ArpRequestResponseMessenger (implementation)
    Package: arp
    File:    ArpRequestResponseMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Hashtable;
import java.util.Vector;

/**
    ArpRequestResponseMessenger
    
    Methods for handling ARP Request responses (Arp replies).
*/
public class ArpRequestResponseMessenger
  implements Messenger {
    public static ArpRequestResponseMessenger instance = new ArpRequestResponseMessenger();

    /**
        Not implemented, planned functionality: gets message from
        requestTable and sets next
        Conduit.
    */
    public void dequeueMessage(int ipAddr, Hashtable reqTable, Session session) {
    //+ ArpRequestResponseMessenger::dequeueMessage(int, Hashtable, Session)
      Vector msgVec = (Vector) reqTable.get(new Integer(ipAddr));
      if (msgVec != null) {
	for( int i = 0; i < msgVec.size() ; i++) {
	  Message msg = (Message) msgVec.elementAt(i);
	  ((ArpRequestSession) session).injectMessage("SideBopposite", msg);
	}
	msgVec.removeAllElements();
	reqTable.remove(new Integer(ipAddr));
      } else {
	if (Arp.DebugLevel > 2) {  // #Debug#
	  System.out.print("ArpRequestResponseMessenger.dequeueMessage: ");
	  System.out.println("nothing to dequeue");
	}
      }
    //- ArpRequestResponseMessenger::dequeueMessage(int, Hashtable, Session)
    }

    /**
        Handles Arp request replies. Inserts IP & Ethernet
        address pair into Arp Table and deletes reply messages.
        
        @param my_session
        @param message
        @return integer
    */
    public int handleArpResponse(ArpRequestSession my_session, Message message) {
    //+ ArpRequestResponseMessenger::handleArpResponse(ArpRequestSession, Message)

      // The metaMsg contains an ARP response (we hope). Go investigate!
      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("Now in ArpRequestResponseMessenger.handleArpResponse");
      }

      int SIZEOFARPMESSAGE = 28;     

      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      Object buf = message.getContents();
      int len = UtilMsgr.i.getLength(buf);

      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("<-- Received an ARP reply.");
      }
      if (Arp.DebugLevel > 2) {  // #Debug#
	System.out.println("<-- ARP reply packet dump:");
	UtilMsgr.i.hexDump(buf);
      }

      long ethSrc;   // These come from the real ethernet header
      long ethDst;
      int frameType;
      long sender_ha;  // These are in the ARP reply,
      int  sender_ip;  // that is, in the ethernet data portion
      long target_ha;
      int  target_ip;

      ethSrc = oob.getSrcEthAddr();
      ethDst = oob.getDestEthAddr();
      frameType = oob.getEthFrameType();

      boolean goodResponse = true;
      String errMsg = new String();

      if ( len < SIZEOFARPMESSAGE ) {
        goodResponse = false;
	errMsg = errMsg + "Packet too small to be and ARP response";
      }
      if (UtilMsgr.i.getShort(buf, 0) != 0x0001) {
        goodResponse = false;
	errMsg = errMsg + "Packet type not 10 Mbit Ethernet";
      }
      if (UtilMsgr.i.getShort(buf, 1) != 0x0800) { // DOD IPv4 = 0x0800
        goodResponse = false;
	errMsg = errMsg + "ARP Packet does not contain an IPv4 mapping";
      }
      if ( UtilMsgr.i.getByte(buf, 4) != 6 ) { // byte length of each hardware address
        goodResponse = false;
	errMsg = errMsg + "ARP response has invalid hardware address length";
      }
      if ( UtilMsgr.i.getByte(buf, 5) != 4 ) { // byte length of each protocol address
        goodResponse = false;
	errMsg = errMsg + "ARP response has invalid protocol address length";
      }

      if ( UtilMsgr.i.getShortUnaligned(buf, 6) != 0x0002) { // ARP Reply = 0x0002
        goodResponse = false;
	errMsg = errMsg + "Internal error: What should have been an ARP " +
	         "respone was not!";
      }

      sender_ha = UtilMsgr.i.get6BytesUnaligned(buf, 8);
      sender_ip = UtilMsgr.i.getIntUnaligned(buf, 14);
      target_ha = UtilMsgr.i.get6BytesUnaligned(buf, 18);
      target_ip = UtilMsgr.i.getIntUnaligned(buf, 24);
      
      if (sender_ha != ethSrc) {
	goodResponse = false;   // Something very fishy going on here..
	errMsg = errMsg + "Somebody else than the owner of the ethernet " +
	  "address responded!";
      }

      // XXX: cms: the following should be implemented:
      // message.stopTransportation(); // Message has been handled and can now be deleted.
  
      if (goodResponse) {
	if (Arp.DebugLevel > 0) {  // #Debug#
	  System.out.print("ARP Reply OK, updating ARP table with: MAC = " +
			   EthernetAddress.toString(sender_ha));
	  System.out.println(", IP = " + IpAddress.toString(sender_ip));
	}	  
	my_session.arpTable.insertEntry(sender_ip, sender_ha, Arp.ARPOK);
	return (sender_ip);
      } else {
	if (Arp.DebugLevel > 0) {  // #Debug#
	  System.out.println("ARP Reply NOT OK: " + errMsg);
	}
	return (-1);
      }

    //- ArpRequestResponseMessenger::handleArpResponse(ArpRequestSession, Message)
    }

    /**
        @param message
        @param state
        @param session
        
    */
    public void apply(Message message, State state, Session session) {
    //+ ArpRequestResponseMessenger::apply(Message, State, Session)

      ArpRequestState arpRequestState = (ArpRequestState) state;

      ArpRequestSession arpRequestSession = (ArpRequestSession) session;

      arpRequestState.applyResponse(this, message, arpRequestSession.requestTable,
				    arpRequestSession ); 

    //- ArpRequestResponseMessenger::apply(Message, State, Session)
    }

}; //ArpRequestResponseMessenger

