/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  XXX   %Z%%M% version %I% date:%G%
  Component:  ArpRequestSession
  Package: arp
  File:    ArpRequestSession.java
*/

package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import java.util.Hashtable;
import fi.hut.tcm.jacob.tcpip.*;

/**
  ArpRequestSession
  
  - Sends ARP request for IP-addresses not found in the ARP Table.
  Inserts found addresses in the ARP table. <br>
  - This session receives packets without next-hop Ethernet addresses, 
  retrieves the addresses, updates the
       ARPTable and sends the packets back to the EtherAddrMUX. <br> 
   - The ARPRequestDataMessenger messenger provides methods for queueing 
  packets and issuing ARP requests:
       enqueueMessage() and makeARPRequest. The entry point in the 
  ARPRequestState for this messenger is
       applyData(). <br>
   - The ARPRequestResponseMessenger provides methods for removing packets 
  from the queue and returning
       them to the EtherAddrMux, UpdateARPTable() and dequeueMessage(). The
   entry point in the
       ARPRequestState for this messenger is applyResponse(). <br>
  - Before generating an ARP failed entry in the ARPTable, the ARP request 
  must be re-attempted at least once. <br>
  - The timeouts use the Conduit timeout mechanism. 
  
*/
class ArpRequestSession
  extends Session {

  /**
    Messages waiting for Arp reply are stored here.
  */
  public Hashtable requestTable;
  
  /**
  */
  public EthernetAddress ethAddr;
  
  /**
  */
  public ArpTable arpTable;
  
  /**
    Constructor for ArpRequestSession.
  */
  public ArpRequestSession(ArpTable arptable, State initialState, EthernetAddress ethAddr) {
  //+ ArpRequestSession::ArpRequestSession(ArpTable, State, EthernetAddress)
      super(initialState);
      this.requestTable = new Hashtable(Arp.RequestTableSize);
      this.ethAddr = ethAddr;
      arpTable = arptable;
  //- ArpRequestSession::ArpRequestSession(ArpTable, State, EthernetAddress)
  }
  
  /**
    Direction = "SideA" or "SideB"
  */
  public void injectMessage(String direction, Message mesg) {
  //+ ArpRequestSession::injectMessage(String, Message)

      if (direction.equals("SideA") ) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sides[Side.A].getAttached(), mesg);
	SimpleScheduler.instance.makeRunnable(mt);

	if (Arp.DebugLevel > 2) {  // #Debug#
	  System.out.println("+++ ArpRequestSession: injected to side A");
	}
      } else if (direction.equals("SideBopposite")) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sides[Side.B].getAttached().getNext().getAttached(),
						   mesg);
	SimpleScheduler.instance.makeRunnable(mt);

	if (Arp.DebugLevel > 2) {  // #Debug#
	  System.out.println("+++ ArpRequestSession: injected to side B");
	}
      } else {
	System.err.println("ArpRequestSession.injectMessage: Error:" +
			   " invalid direction: " + direction);
      }
     
  //- ArpRequestSession::injectMessage(String, Message)
  }
  
} // ArpRequestSession

// +*************** UNUSED CODE ***************

// -*************** UNUSED CODE ***************
