/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  ArpRequestState (implementation)
    Package: arp
    File:    ArpRequestState.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Hashtable;

/**
    ArpRequestState
    
    Provides methods for sending ARP requests and handling replies: <br>
    - applyData calls APRequestDataMessenger to make
    ARP request to get Ethernet address for packet. <br>
    - applyResponse calls ARPRequestResponseMessenger to
    insert the IP-Ethernet pair in response message to
    the Arp Table.
*/
public class ArpRequestState
  implements State {
    public static ArpRequestState instance = new ArpRequestState();

    /**
        Calls ARPRequestResponseMessenger to
        insert the IP-Ethernet pair in response message to
        the Arp Table
        
    */
    public void applyResponse(Messenger msgr, Message message, Hashtable reqTable, ArpRequestSession my_session) {
    //+ ArpRequestState::applyResponse(Messenger, Message, Hashtable, ArpRequestSession)

      ArpRequestResponseMessenger messenger =
	(ArpRequestResponseMessenger) msgr;
    
      int ipAddress;
    
      if ( (ipAddress = messenger.handleArpResponse(my_session, message )) != -1) {
	messenger.dequeueMessage( ipAddress, reqTable, my_session );
      }
      // The response should not continue travelling
      my_session.releaseMessage(message);
    
    //- ArpRequestState::applyResponse(Messenger, Message, Hashtable, ArpRequestSession)
    }

    /**
        Calls APRequestDataMessenger to make
        ARP request to get Ethernet address for packet.
        
        @param msgr
        @param message
        @param reqTable
        @param ifaceTable
        @param my_session
        
    */
    public void applyData(Messenger msgr, Message message, Hashtable reqTable, ArpRequestSession my_session) {
    //+ ArpRequestState::applyData(Messenger, Message, Hashtable, ArpRequestSession)

      ArpRequestDataMessenger messenger =
	(ArpRequestDataMessenger) msgr;
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      messenger.enqueueMessage( message, reqTable );
      my_session.stopMessage( message );
      messenger.makeArpRequest( message, oob.getNextHopIpAddr(), my_session );

    //- ArpRequestState::applyData(Messenger, Message, Hashtable, ArpRequestSession)
    }

}; //ArpRequestState

