/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  XXX   %Z%%M% version %I% date:%G%
  Component:  EtherAddrAccessor
  Package: arp
  File:    EtherAddrAccessor.java
*/

package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;



/**
  EtherAddrAccessor
  
  Used by EtherAddrMUX to multiplex IP packets.
*/
class EtherAddrAccessor
  extends Object
  implements Accessor {

  /**
  */
  protected ArpTable arpTable;
  
  /**
    Sets the key for a message.
    
    @param    message
    @param    key
  */
  public void setKey(Message message, Object key) {
  //+ EtherAddrAccessor::setKey(Message, Object)
      //  Nothing to do here.
  //- EtherAddrAccessor::setKey(Message, Object)
  }
  
  /**
  */
  public void unsetKey(Message message) {
  //+ EtherAddrAccessor::unsetKey(Message)
    // nothing to see here!
  //- EtherAddrAccessor::unsetKey(Message)
  }
  
  /**
    Returns key for multiplexing IP packets:<br>
    "1"  OK messages IP address was found from Arp Table. Forwarded to 
    ARPSelectorMUX.<br>
    "2"  ARP NOT IN TABLE, IP address was not found
    from Arp Table and Arp Request must be done.
    Forwarded to ArpRequestSession.<br>
    "3" ARP FAILURE, IP address was not found from
    Arp Table and Arp Request failed too. 
    Forwarded to ICMP adaptor for generating error
    for application. 
    
    @param    message
    @return   Object   returns the corresponding key
  */
  public Object getKey(Message message) {
  //+ EtherAddrAccessor::getKey(Message)
    
    long newEtherAddr;

    TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();  
    Object buf = message.getContents();

    if ( (newEtherAddr = arpTable.getEtherAddr(oob.getNextHopIpAddr())) >= 0) {
      oob.setDestEthAddr(newEtherAddr);
      oob.setEthFrameType( 0x0800 );
      message.setMessenger(EthernetHeaderToNetMessenger.instance);
      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("EtherAddrAccessor - OK - sent to ArpSelectorMUX");
      }
      return("1");
    }  

    // There was an error. Use a new variable, so that we remember..
    int errorcode = (int) newEtherAddr;

    if(errorcode == Arp.ARPNOTINTABLE) {
      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("EtherAddrAccessor - ARPNOTINTABLE - sent to RequestSession");
      }
      message.setMessenger(ArpRequestDataMessenger.instance);
      return("2");
    }

    if(errorcode == Arp.ARPFAILURE) {
      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("EtherAddrAccessor - ARPFAILURE - sent to ICMP adaptor");
      }
      return( null );
    } 

    // Should not arrive here - ever.
    if (Arp.DebugLevel > 0) {  // #Debug#
      System.out.println("Internal oddity in EtherAddrAccessor.getKey - unknown key");
    }
    return( null );

  //- EtherAddrAccessor::getKey(Message)
  }
  
  /**
  */
  public EtherAddrAccessor(ArpTable arptable) {
  //+ EtherAddrAccessor::EtherAddrAccessor(ArpTable)
      arpTable = arptable;
  //- EtherAddrAccessor::EtherAddrAccessor(ArpTable)
  }
  
} // EtherAddrAccessor

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
