/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  EthernetHeaderFromNetMessenger (implementation)
    Package: arp
    File:    EthernetHeaderFromNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    EthernetHeaderFromNetMessenger
    
    Provides the method removeEthernetHeader for removing Ethernet header
    from Message.
*/
public class EthernetHeaderFromNetMessenger
  implements Messenger {
    public static EthernetHeaderFromNetMessenger instance = new EthernetHeaderFromNetMessenger();

    /**
        Removes Ethernet header from message and
        inserts Ethernet header type information to
        out of band part of the message.
        @param message
    */
    public void removeEthernetHeader(Message message) {
    //+ EthernetHeaderFromNetMessenger::removeEthernetHeader(Message)

      Object buf = message.getContents();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();

      long ethDst = UtilMsgr.i.get6BytesUnaligned(buf, 0);
      long ethSrc = UtilMsgr.i.get6BytesUnaligned(buf, 6);
      int frType = UtilMsgr.i.getShort(buf, 6);

      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.print("EthernetHeaderFromNetMessenger: ethSrc = " +
			 EthernetAddress.toString(ethSrc));
	System.out.println(", ethDst = " + EthernetAddress.toString(ethDst) );
      }

      oob.setDestEthAddr(ethDst);
      oob.setSrcEthAddr(ethSrc);
      oob.setEthFrameType(frType);

      if (Arp.DebugLevel > 1) {  // #Debug#
	System.out.println("Packet dump:");
	UtilMsgr.i.hexDump(buf);
      }
      
      UtilMsgr.i.removeHeader(buf, Arp.SIZEOFETHERHEADER);
    //- EthernetHeaderFromNetMessenger::removeEthernetHeader(Message)
    }

    /**
        @param message
        @param state
        @param session
    */
    public void apply(Message message, State state, Session session) {
    //+ EthernetHeaderFromNetMessenger::apply(Message, State, Session)
      EthernetHeaderState my_state = (EthernetHeaderState) state;
      my_state.applyFromNet( this, message );
    //- EthernetHeaderFromNetMessenger::apply(Message, State, Session)
    }

}; //EthernetHeaderFromNetMessenger

