/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  EthernetHeaderState (implementation)
    Package: arp
    File:    EthernetHeaderState.java


*/
package fi.hut.tcm.jacob.tcpip.arp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    EthernetHeaderState
    
    Methods for handling Ethernet header of incoming and outgoing packets:
    <br>
    - applyFromNet calls EthernetFromNetMessenger.removeEthernetHeader
    to remove Ethernet header from in coming pakects. <br>
    - applyToNet calls EthernetToNetMessenger.addEtherNetHeader
    to create Ethernet header for out going packets.
*/
public class EthernetHeaderState
  implements State {
    public static EthernetHeaderState instance = new EthernetHeaderState();

    /**
        Calls EthernetFromNetMessenger.removeEthernetHeader
        to remove Ethernet header from in coming pakects.
        @param messenger
        @param message
    */
    public void applyFromNet(Messenger messenger, Message message) {
    //+ EthernetHeaderState::applyFromNet(Messenger, Message)
      //if (messenger instanceof EthernetFromNetMessenger)
      EthernetHeaderFromNetMessenger msgr = 
	(EthernetHeaderFromNetMessenger) messenger;
      
      // Does more than removes the header: also inserts data into oob:
      msgr.removeEthernetHeader(message);

      // There is no sensible Messenger to attach to the Message. The next
      // conduit, the ArpSelectorMUX (really its accessor) handles that.

    //- EthernetHeaderState::applyFromNet(Messenger, Message)
    }

    /**
        calls EthernetToNetMessenger.addEtherNetHeader
        to create Ethernet header for out going packets.
        
        @param messenger
        @param message
    */
    public void applyToNet(Messenger messenger, Message message) {
    //+ EthernetHeaderState::applyToNet(Messenger, Message)
      //      if (messenger instanceof EthernetToNetMessenger)
      EthernetHeaderToNetMessenger msgr = 
	(EthernetHeaderToNetMessenger) messenger;


      // Adds the physical header: and inserts the destination
      // ethernet address, which can be found in the oob:
      msgr.addEthernetHeader(message);

    //- EthernetHeaderState::applyToNet(Messenger, Message)
    }

}; //EthernetHeaderState

