/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpChecksumFromNetMessenger (implementation)
    Package: icmp
    File:    IcmpChecksumFromNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpChecksumFromNetMessenger
    
    <p>The Messenger containing the logic for incoming ICMP messages. This
    Messenger applies itself to the <code>IcmpChecksumState</code>.
    Contains a method for validating the ICMP checksum.
*/
public class IcmpChecksumFromNetMessenger
  extends IcmpChecksumMessenger
  implements Messenger {
    /**
        Only one <code>Messenger</code> of this particular kind should
        exist.
    */
    public static IcmpChecksumFromNetMessenger instance = new IcmpChecksumFromNetMessenger();

    /**
        Applies a message to the state of a session
        
        @param message  <code>Message</code>
        @param state  <code>State</code>
        @param session <code>Session</code>
    */
    public void apply(Message message, State state, Session session) {
    //+ IcmpChecksumFromNetMessenger::apply(Message, State, Session)
      IcmpChecksumState stat = (IcmpChecksumState) state;
      if (! stat.applyFromNet(message, this) ) {
	session.releaseMessage(message);
      }
    //- IcmpChecksumFromNetMessenger::apply(Message, State, Session)
    }

    /**
        Validates the ICMP checksum of a message.
                
        @param message which <code>Message</code> to use
        @return    <code>true</code>, if the checksum was ok
    */
    public boolean validChecksum(Message message) {
    //+ IcmpChecksumFromNetMessenger::validChecksum(Message)
      return( calculateChecksum(message) == getChecksum(message) );
    //- IcmpChecksumFromNetMessenger::validChecksum(Message)
    }

}; //IcmpChecksumFromNetMessenger

