/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpChecksumToNetMessenger (implementation)
    Package: icmp
    File:    IcmpChecksumToNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpChecksumToNetMessenger
    
    <p>This is the messenger that contains the logic for ICMP messages
    going out to the network. Contains a method for calculating the
    checksum of this ICMP message and adds it to the message.
*/
public class IcmpChecksumToNetMessenger
  extends IcmpChecksumMessenger
  implements Messenger {
    /**
        There should exist only one <code>Messenger</code> of this kind.
    */
    public static IcmpChecksumToNetMessenger instance = new IcmpChecksumToNetMessenger();

    /**
        Method for applying a message to a state of a specific session.
                
        @param message  <code>Message</code>
        @param state  <code>State</code>
        @param session <code>Session</code>
    */
    public void apply(Message message, State state, Session session) {
    //+ IcmpChecksumToNetMessenger::apply(Message, State, Session)
      if(Icmp.DebugLevel > 3)
	System.out.println("IcmpChecksumToNetMessenger.apply()");
      IcmpChecksumState stat = (IcmpChecksumState) state;
      stat.applyToNet(message, this);
    //- IcmpChecksumToNetMessenger::apply(Message, State, Session)
    }

    /**
        Calculates and adds the ICMP checksum to a message.
                
        @param message <code>Message</code>
    */
    public void addChecksum(Message message) {
    //+ IcmpChecksumToNetMessenger::addChecksum(Message)
      if(Icmp.DebugLevel > 3)
	System.out.println("IcmpChecksumToNetMessenger.addChecksum()");
      setChecksum(message, calculateChecksum(message));
    //- IcmpChecksumToNetMessenger::addChecksum(Message)
    }

}; //IcmpChecksumToNetMessenger

