/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpTypeAccessor (implementation)
    Package: icmp
    File:    IcmpTypeAccessor.java


*/
package fi.hut.tcm.jacob.tcpip.icmp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IcmpTypeAccessor
    
    <p>This accessor examines the ICMP type of the message. The key is the
    same as the ICMP type for all ICMP types except ICMP Echo and ICMP Echo
    Reply. The reason for this is that incoming and outgoing Echo Requests
    (and replies) have different routes.
*/
public class IcmpTypeAccessor
  implements Accessor {

    /**
        Gets the key of a <code>Message</code>, in order to be able to
        route it correctly. In this version though, it does not yet do it,
        only outputs debug information.
        
        @param    message  which <code>Message</code>
        @return    returns the corresponding key
    */
    public Object getKey(Message message) {
    //+ IcmpTypeAccessor::getKey(Message)

      Object buf = message.getContents();
      int len = IcmpUtilityMessenger.instance.getLength(buf); 
      int type = IcmpUtilityMessenger.instance.getByte(buf, 0);
      if (Icmp.DebugLevel > 2) {  // #Debug#
	System.out.println("In IcmpTypeAccessor.getKey, key is " + type);
      }
      return new Integer(type);
    //- IcmpTypeAccessor::getKey(Message)
    }

    /**
        Sets the key for a message, but right now, does nothing but print
        debug output.
        
        @param    message  which <code>Message</code>
        @param    key  the key for the <code>Message</code>
    */
    public void setKey(Message message, Object key) {
    //+ IcmpTypeAccessor::setKey(Message, Object)

      Object buf = message.getContents();
      int type = ((Integer) key).intValue();
      // Ugly hack: here we reverse the types for Echo and Echo Reply:
      if (type == 0) {
	type = 8;
      } else if (type == 8) {
	type = 0;
      }
      if(Icmp.DebugLevel > 3)
	System.out.println("flipping..., type now " + type);
      IcmpUtilityMessenger.instance.setByte(buf, 0, type);
      message.setMessenger(IcmpChecksumToNetMessenger.instance);
    //- IcmpTypeAccessor::setKey(Message, Object)
    }


    /**

    */
    public void unsetKey(Message message) {
    //+ IcmpTypeAccessor::unsetKey(Message)
      // Do nothing...
      if (Icmp.DebugLevel > 2) {  // #Debug#
	System.out.println("Now in IcmpTypeAccessor.unsetKey()");
      }
    //- IcmpTypeAccessor::unsetKey(Message)
    }

}; //IcmpTypeAccessor

