/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  IP
  Package: ip
  File:    IP.java
*/

package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
  IP
  
  
*/
public class IP
  extends Protocol {

  /**
  */
  static final int IP_VERSION = 4;
  
  /**
  */
  static final int IP_PROTOCOL_ID = 1;
  
  /**
  */
  private TcpipRoutingTable rTable = null;
  
  /**
  */
  private TcpipInterfaceTable ifTable = null;
  
  /**
  */
  private Mux interfaceMux = null;
  
  /**
  */
  private Mux protocolMux = null;
  
  /**
  */
  public IP(TcpipRoutingTable rTable, TcpipInterfaceTable ifTable)
    throws SideException  {
  
  //+ IP::IP(TcpipRoutingTable, TcpipInterfaceTable)
    super();

    if (rTable == null) {
      throw new IllegalArgumentException("IP must be initialized with a non-null routing table");
    }
    if (ifTable == null) {
      throw new IllegalArgumentException("IP must be initialized with a non-null interface table");
    }

    this.rTable = rTable;
    this.ifTable = ifTable;

    IpRoutingAccessor ipRAcc = new IpRoutingAccessor();
    interfaceMux = new Mux(ipRAcc);
    Terminator ipRTer = new Terminator();
    IpRoutingSession ipRSes =
      new IpRoutingSession(IpRoutingOffState.instance, rTable, ifTable);
    IpReassemblySession ipReSes = 
      new IpReassemblySession(IpReassemblyState.instance);
    IpProtocolAccessor ipPAcc = new IpProtocolAccessor();
    protocolMux = new Mux(ipPAcc);
    Terminator ipPTer = new Terminator();
    
    try {
      ipRTer.getSide(Side.A).attach(interfaceMux.getSide(Side.B));
      interfaceMux.getSide(Side.A).attach(ipRSes.getSide(Side.A));
      ipRSes.getSide(Side.B).attach(ipReSes.getSide(Side.A));
      ipReSes.getSide(Side.B).attach(protocolMux.getSide(Side.A));
      protocolMux.getSide(Side.B).attach(ipPTer.getSide(Side.A));
    } catch (SideException e) {
      throw e;
    }
  //- IP::IP(TcpipRoutingTable, TcpipInterfaceTable)
  }
  
  /**
  */
  public void addInterface(Side endpoint, String name)
    throws SideException  {
  
  //+ IP::addInterface(Side, String)
    IpChecksumSession ipCSes =
      new IpChecksumSession(IpChecksumState.instance);
    IpFragmentationSession ipFrSes =
      new IpFragmentationSession(IpFragmentationState.instance, ifTable);
    
    try {
      endpoint.attach(ipCSes.getSide(Side.A));
      ipCSes.getSide(Side.B).attach(ipFrSes.getSide(Side.A));
      ipFrSes.getSide(Side.B).attach(interfaceMux.createSide(name));
    } catch (SideException e) {
      throw e;
    }
  //- IP::addInterface(Side, String)
  }
  
  /**
  */
  public void removeInterface(String name) {
  
  //+ IP::removeInterface(String)
  //- IP::removeInterface(String)
  }
  
  /**
  */
  public void addProtocol(Side endpoint, byte id)
    throws SideException  {
  
  //+ IP::addProtocol(Side, byte)
    IpHeaderSession ipHSes = new IpHeaderSession(IpHeaderState.instance);

    try {
      protocolMux.createSide(new Byte(id)).attach(ipHSes.getSide(Side.A));
      ipHSes.getSide(Side.B).attach(endpoint);
    } catch (SideException e) {
      throw e;
    }
  //- IP::addProtocol(Side, byte)
  }
  
  /**
  */
  public void removeProtocol(byte id) {
  
  //+ IP::removeProtocol(byte)
  //- IP::removeProtocol(byte)
  }
  
} // IP

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
