/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpChecksumFromNetMessenger (implementation)
    Package: ip
    File:    IpChecksumFromNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpChecksumFromNetMessenger
    
    
*/
public class IpChecksumFromNetMessenger
  extends IpMessenger {
    public static IpChecksumFromNetMessenger instance = new IpChecksumFromNetMessenger();

    public void apply(Message message, State state, Session session) {
    //+ IpChecksumFromNetMessenger::apply(Message, State, Session)
      IpChecksumState st = (IpChecksumState) state;
      st.applyFromNet(message, session, this);
    //- IpChecksumFromNetMessenger::apply(Message, State, Session)
    }

    /**
        Validates the checksum
    */
    public boolean validChecksum(Message message) {
    //+ IpChecksumFromNetMessenger::validChecksum(Message)
      Object buffer = message.getContents();
      byte hlen = (byte) getHLen(message);
      int csum_field = getChecksum(message);

      // DEBUG:
      //System.out.println("IpChecksumFromNetMessenger: hlen is " + hlen);
      //System.out.println("Message's total length is: " + UtilMsgr.i.getLength(buffer));

      // The csum field in IP header must be zero before verification
      setChecksum(message, 0);

      int csum = 0;
      for (int i = hlen * 2 - 1; i >= 0; i -= 1) {
	csum += ((int) getShort(buffer, i)) & 0xffff;
      }
      /* 32-bits to 16-bit folding and 1's complement */
      while ((csum >> 16) > 0) {
	csum = (csum & 0xffff) + ((csum >> 16) & 0xffff);
      }
      csum = ~csum & 0xffff;

      if (csum == csum_field) {
	return true;
      }
      else {
	System.err.println("Checksum field " + csum_field + " should be " +
			   csum);
	return false;
      }
      //- IpChecksumFromNetMessenger::validChecksum(Message)
    }

}; //IpChecksumFromNetMessenger

