/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpChecksumToNetMessenger (implementation)
    Package: ip
    File:    IpChecksumToNetMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpChecksumToNetMessenger
    
    
*/
public class IpChecksumToNetMessenger
  extends IpMessenger {
    public static IpChecksumToNetMessenger instance = new IpChecksumToNetMessenger();

    public void apply(Message message, State state, Session session) {
    //+ IpChecksumToNetMessenger::apply(Message, State, Session)
      IpChecksumState st = (IpChecksumState) state;
      st.applyToNet(message, session, this);
    //- IpChecksumToNetMessenger::apply(Message, State, Session)
    }

    /**
        Calculates the IP header checksum and adds it to the packet.
    */
    public int calculateChecksum(Message message) {
    //+ IpChecksumToNetMessenger::calculateChecksum(Message)
      Object buffer = message.getContents();
      byte hlen = (byte) getHLen(message);

      setChecksum(message, 0);

      int csum = 0;
      for (int i = hlen * 2 - 1; i >= 0; i -= 1) {
	csum += ((int) getShort(buffer, i)) & 0xffff;
      }
      /* 32-bit to 16-bit folding and 1's complement */
      while ((csum >> 16) > 0) {
	csum = (csum & 0xffff) + ((csum >> 16) & 0xffff);
      }
      csum = ~csum & 0xffff;
      
      setChecksum(message, csum);
      return csum;
    //- IpChecksumToNetMessenger::calculateChecksum(Message)
    }

}; //IpChecksumToNetMessenger
