/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  IpFragmentationToNetMessenger
  Package: ip
  File:    IpFragmentationToNetMessenger.java
*/

package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.util.Enumeration;
import java.util.Vector;

/**
  IpFragmentationToNetMessenger
  
  Calls the state's applyToNet() method.
  
*/
class IpFragmentationToNetMessenger
  extends IpMessenger {

  /**
  */
  public static IpFragmentationToNetMessenger instance = new IpFragmentationToNetMessenger();
  
  /**
  */
  public void apply(Message message, State state, Session session) {
  
  //+ IpFragmentationToNetMessenger::apply(Message, State, Session)
      IpFragmentationState st = (IpFragmentationState) state;
      st.applyToNet(message, session, this);
  //- IpFragmentationToNetMessenger::apply(Message, State, Session)
  }
  
  /**
    Does the hard work of fragmenting those IP packets that are longer than
     the MTU.
    
  */
  public Vector fragment(Message message, int mtu, Session session) {
  
  //+ IpFragmentationToNetMessenger::fragment(Message, int, Session)
    Object buffer = message.getContents();
    TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
    
    Vector fragments = new Vector();
    int tlen = getTotalLength(message);
    
    if (tlen > mtu) {
      /* 
       * Fragmentation needed 
       */
      if ((getFlags(message) & 0x02) != 0) {
	/* Don't fragment bit set */
	// XXX generate ICMP message
      } else {
	/* Fragment datagram */
	int hlen = getHLen(message) * 4;
	mtu -= (mtu - hlen) % 8; // Frag. offsets are multiples of 8 octets
	System.err.println("Dividing " + tlen + " bytes to " + mtu + 
			   " byte fragments");
	int offs = hlen;
	
	while (offs < tlen) {
	  
	  Object newBuffer = newBuffer(mtu, 0, mtu);
	  copyBuffer(buffer, 0, newBuffer, 0, hlen);
	  int datalen = (mtu - hlen < tlen - offs)? mtu - hlen : tlen - offs;
	  copyBuffer(buffer, offs, newBuffer, hlen, datalen);
	  
	  Message newMessage = 
	    Message.newMessage(newBuffer,
			       IpChecksumToNetMessenger.instance, 
			       message.getOob());
	  
	  // Fill in fragment specific data
	  setTotalLength(newMessage, datalen + hlen);
	  setFragOffs(newMessage, (offs - hlen) / 8);
	  offs += datalen;
	  if (offs < tlen) {
	    setFlags(newMessage, getFlags(message) | 0x01);
	  }
	  fragments.addElement(newMessage);
	}
      }
      // The fragmented datagram is discarded
      ((IpFragmentationSession)session).releaseMessage(message);
    }
    
    return fragments;
    //- IpFragmentationToNetMessenger::fragment(Message, int, Session)
  }
  
  /**
  */
  public String getInterfaceId(Message message) {
  
  //+ IpFragmentationToNetMessenger::getInterfaceId(Message)
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      return oob.getInterfaceId();
  //- IpFragmentationToNetMessenger::getInterfaceId(Message)
  }
  
} // IpFragmentationToNetMessenger

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
