/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  IpMessenger
  Package: ip
  File:    IpMessenger.java
*/

package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
  IpMessenger
  
  The base class of all Messengers for the IP protocol. Implements the 
  methods for manipulating IP headers.
*/
abstract class IpMessenger
  extends BufferMessenger
  implements Messenger {

  /**
  */
  public int getVers(Message message) {
  
  //+ IpMessenger::getVers(Message)
      Object buffer = message.getContents();
      return (getByte(buffer, 0) >> 4) & 0x0f;
  //- IpMessenger::getVers(Message)
  }
  
  /**
  */
  public void setVers(Message message, int vers) {
  
  //+ IpMessenger::setVers(Message, int)
      Object buffer = message.getContents();
      int bufdata = getByte(buffer, 0);
      setByte(buffer, 0, ((vers & 0x0f) << 4) | (bufdata & 0x0f));
  //- IpMessenger::setVers(Message, int)
  }
  
  /**
  */
  public int getHLen(Message message) {
  
  //+ IpMessenger::getHLen(Message)
      Object buffer = message.getContents();
      return getByte(buffer, 0) & 0x0f;
  //- IpMessenger::getHLen(Message)
  }
  
  /**
  */
  public void setHLen(Message message, int hlen) {
  
  //+ IpMessenger::setHLen(Message, int)
      Object buffer = message.getContents();
      int bufdata = getByte(buffer, 0);
      setByte(buffer, 0, (hlen & 0x0f) | (bufdata & 0xf0));
  //- IpMessenger::setHLen(Message, int)
  }
  
  /**
  */
  public int getServiceType(Message message) {
  
  //+ IpMessenger::getServiceType(Message)
      Object buffer = message.getContents();
      return getByte(buffer, 1);
  //- IpMessenger::getServiceType(Message)
  }
  
  /**
  */
  public void setServiceType(Message message, int stype) {
  
  //+ IpMessenger::setServiceType(Message, int)
      Object buffer = message.getContents();
      setByte(buffer, 1, stype);
  //- IpMessenger::setServiceType(Message, int)
  }
  
  /**
  */
  public int getTotalLength(Message message) {
  
  //+ IpMessenger::getTotalLength(Message)
      Object buffer = message.getContents();
      return getShort(buffer, 1);
  //- IpMessenger::getTotalLength(Message)
  }
  
  /**
  */
  public void setTotalLength(Message message, int len) {
  
  //+ IpMessenger::setTotalLength(Message, int)
      Object buffer = message.getContents();
      setShort(buffer, 1, len);
  //- IpMessenger::setTotalLength(Message, int)
  }
  
  /**
  */
  public int getIdentification(Message message) {
  
  //+ IpMessenger::getIdentification(Message)
      Object buffer = message.getContents();
      return getShort(buffer, 2);
  //- IpMessenger::getIdentification(Message)
  }
  
  /**
  */
  public void setIdentification(Message message, int id) {
  
  //+ IpMessenger::setIdentification(Message, int)
      Object buffer = message.getContents();
      setShort(buffer, 2, id);
  //- IpMessenger::setIdentification(Message, int)
  }
  
  /**
  */
  public int getFlags(Message message) {
  
  //+ IpMessenger::getFlags(Message)
      Object buffer = message.getContents();
      return (getByte(buffer, 6) >> 5) & 0x07;
  //- IpMessenger::getFlags(Message)
  }
  
  /**
  */
  public void setFlags(Message message, int flags) {
  
  //+ IpMessenger::setFlags(Message, int)
      Object buffer = message.getContents();
      int bufdata = ((int) getByte(buffer, 6)) & 0x1f;
      setByte(buffer, 6, ((flags << 5) | bufdata));
  //- IpMessenger::setFlags(Message, int)
  }
  
  /**
  */
  public int getFragOffs(Message message) {
  
  //+ IpMessenger::getFragOffs(Message)
      Object buffer = message.getContents();
      return getShort(buffer, 3) & 0x1fff;
  //- IpMessenger::getFragOffs(Message)
  }
  
  /**
  */
  public void setFragOffs(Message message, int offs) {
  
  //+ IpMessenger::setFragOffs(Message, int)
      Object buffer = message.getContents();
      int bufdata = ((int) getShort(buffer, 3)) & 0xe000;
      setShort(buffer, 3, (bufdata | offs));
  //- IpMessenger::setFragOffs(Message, int)
  }
  
  /**
  */
  public int getTTL(Message message) {
  
  //+ IpMessenger::getTTL(Message)
      Object buffer = message.getContents();
      return getByte(buffer, 8);
  //- IpMessenger::getTTL(Message)
  }
  
  /**
  */
  public void setTTL(Message message, int ttl) {
  
  //+ IpMessenger::setTTL(Message, int)
      Object buffer = message.getContents();
      setByte(buffer, 8, ttl);
  //- IpMessenger::setTTL(Message, int)
  }
  
  /**
  */
  public int getProtocolId(Message message) {
  
  //+ IpMessenger::getProtocolId(Message)
      Object buffer = message.getContents();
      return getByte(buffer, 9);
  //- IpMessenger::getProtocolId(Message)
  }
  
  /**
  */
  public void setProtocolId(Message message, int prot) {
  
  //+ IpMessenger::setProtocolId(Message, int)
      Object buffer = message.getContents();
      setByte(buffer, 9, prot);
  //- IpMessenger::setProtocolId(Message, int)
  }
  
  /**
  */
  public int getChecksum(Message message) {
  
  //+ IpMessenger::getChecksum(Message)
      Object buffer = message.getContents();
      return getShort(buffer, 5);
  //- IpMessenger::getChecksum(Message)
  }
  
  /**
  */
  public void setChecksum(Message message, int csum) {
  
  //+ IpMessenger::setChecksum(Message, int)
      Object buffer = message.getContents();
      setShort(buffer, 5, csum);
  //- IpMessenger::setChecksum(Message, int)
  }
  
  /**
  */
  public int getSrcIpAddr(Message message) {
  
  //+ IpMessenger::getSrcIpAddr(Message)
      Object buffer = message.getContents();
      return getInt(buffer, 3);
  //- IpMessenger::getSrcIpAddr(Message)
  }
  
  /**
  */
  public void setSrcIpAddr(Message message, int ipAddr) {
  
  //+ IpMessenger::setSrcIpAddr(Message, int)
      Object buffer = message.getContents();
      setInt(buffer, 3, ipAddr);
  //- IpMessenger::setSrcIpAddr(Message, int)
  }
  
  /**
  */
  public int getDestIpAddr(Message message) {
  
  //+ IpMessenger::getDestIpAddr(Message)
      Object buffer = message.getContents();
      return getInt(buffer, 4);
  //- IpMessenger::getDestIpAddr(Message)
  }
  
  /**
  */
  public void setDestIpAddr(Message message, int ipAddr) {
  
  //+ IpMessenger::setDestIpAddr(Message, int)
      Object buffer = message.getContents();
      setInt(buffer, 4, ipAddr);
  //- IpMessenger::setDestIpAddr(Message, int)
  }
  
} // IpMessenger

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
