/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpReassemblyState (implementation)
    Package: ip
    File:    IpReassemblyState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;

/**
    IpReassemblyState
    
    A state responsible of reassembling fragmented incoming IP packets.
    Ignores outgoing packets, except for updating their messenger
    reference..
*/
public class IpReassemblyState
  implements State {

  public static final IpReassemblyState instance = new IpReassemblyState();

    /**
        Called by IpReassemblyFromNetMessenger. This method in turn calls
        the messenger's reassemble() method which does all the work
    */
    public void applyFromNet(Message message, Session session,
			     Messenger messenger) {
    //+ IpReassemblyState::applyFromNet(Message, Session, Messenger)
      Message newMessage;
      Side nextSide = message.getSourceSide().getNext();
      IpReassemblySession sess = (IpReassemblySession) session;
      IpReassemblyFromNetMessenger msgr =
	(IpReassemblyFromNetMessenger) messenger;
      newMessage = msgr.reassemble(message, session);
      if (newMessage != null) {
	message = newMessage;
	MessageTransporter trans = 
	  MessageTransporter.newMessageTransporter(nextSide, message);
	SimpleScheduler.instance.makeRunnable(trans);
      }
      message.setMessenger(IpHeaderFromNetMessenger.instance);
    //- IpReassemblyState::applyFromNet(Message, Session, Messenger)
    }

    /**
        Called by ReassemblyToNetMessenger. Does not call back on the
        messenger that called it. Instead it just sets the message's
        messenger pointer to point to the next messenger. (Yes, we are
        members of the Humphrey Appleby Fan Club)
        
    */
    public void applyToNet(Message message, Session session, Messenger messenger) {
    //+ IpReassemblyState::applyToNet(Message, Session, Messenger)
      message.setMessenger(IpRoutingToNetMessenger.instance);
    //- IpReassemblyState::applyToNet(Message, Session, Messenger)
    }

}; //IpReassemblyState

