/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpRoutingMessenger (implementation)
    Package: ip
    File:    IpRoutingMessenger.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;


/**
    IpRoutingMessenger
    
    This is the superclass of IpRoutingToNetMessenger and
    IpRoutingFromNetMessenger capturing their common methods. Provides the
    methods:
    1. setNextHopIpAddr()
    Retreives the next hop IP address based on the packet's destination
    address, and puts it into the message's OutOfBand data.
    2. destinationLocal()
    Tells whether the destination IP address is one of our local addresses.
    3. decrementTTL()
    Decrements the TTL field in the IP header, and returns the new value.
    
*/
public abstract class IpRoutingMessenger
  extends IpMessenger {

    public void setNextHopIpAddr(Message message, int ipAddr) {
    //+ IpRoutingMessenger::setNextHopIpAddr(Message, int)
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      oob.setNextHopIpAddr(ipAddr);
    //- IpRoutingMessenger::setNextHopIpAddr(Message, int)
    }

    public void setInterfaceId(Message message, String id) {
    //+ IpRoutingMessenger::setInterfaceId(Message, String)
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      oob.setInterfaceId(id);
    //- IpRoutingMessenger::setInterfaceId(Message, String)
    }

    public boolean destinationLocal(Message message, Session session) {
    //+ IpRoutingMessenger::destinationLocal(Message, Session)
      IpRoutingSession sess = (IpRoutingSession) session;
      int dest = getDestIpAddr(message);
      if (sess.interfaceTable.get(dest) == null) {
	return false;
      }
      return true;
    //- IpRoutingMessenger::destinationLocal(Message, Session)
    }

    public int decrementTTL(Message message) {
    //+ IpRoutingMessenger::decrementTTL(Message)
      Object buffer = message.getContents();
      byte newTTL = (byte) (getByte(buffer, 8) - 1);
      // XXX Is there a need for additional timer-based decrement?
      setByte(buffer, 8, newTTL);
      
      return (int) newTTL & 0xff;
    //- IpRoutingMessenger::decrementTTL(Message)
    }

}; //IpRoutingMessenger
