/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpRoutingOffState (implementation)
    Package: ip
    File:    IpRoutingOffState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpRoutingOffState
    
    The RoutingSession is in this state when routing is disabled.
*/
public class IpRoutingOffState
  extends IpRoutingState {

  public static final IpRoutingOffState instance = new IpRoutingOffState();


    /**
        Calls the Messenger's destinationLocal() method which tells if the
        destination IP address belongs to this machine. If the destination
        is local, then the message is forwarded to the ReassemblySession.
        Packets to nonlocal destinations are destroyed.
        
    */
    public void applyFromNet(Message message, Session session, Messenger messenger) {
    //+ IpRoutingOffState::applyFromNet(Message, Session, Messenger)
      IpRoutingFromNetMessenger msgr = (IpRoutingFromNetMessenger) messenger;
      if (!(msgr.destinationLocal(message, session))) {
	int destIpAddr = msgr.getDestIpAddr(message);
	System.err.println("Packet forwarding disabled.");
	System.err.println("Discarding datagram addressed to " +
			   (((int) destIpAddr >> 24) & 0xff) + "." +
			   (((int) destIpAddr >> 16) & 0xff) + "." +
			   (((int) destIpAddr >>  8) & 0xff) + "." +
			   (((int) destIpAddr >>  0) & 0xff));
	// Just discard it:
	((IpRoutingSession)session).releaseMessage(message);
      }
      message.setMessenger(IpReassemblyFromNetMessenger.instance);
    //- IpRoutingOffState::applyFromNet(Message, Session, Messenger)
    }

}; //IpRoutingOffState

