/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  Copyright (C) 1998 XYZ, Inc. All rights Reserved.
  XXX   %Z%%M% version %I% date:%G%
  Component:  IpRoutingSession
  Package: ip
  File:    IpRoutingSession.java
*/

package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
  IpRoutingSession
  
  1. For outgoing packets, retrieves the next-hop IP from the RoutingTable 
  and puts it in the Out-of-Band data.
  2. Asks the InterfaceTable if the destination IP address is a local one (
  ie belongs to the host). In that case, the IP packet is returned back up 
  to the ReassemblySession. Otherwise, the packet is forwarded to the 
  RoutingMUX.
  3. For incoming packets, if the destination IP belongs to the host, the 
  packet is forwarded to the ReassemblySession.
  4. If the host is not a router and the destination address is not ours, 
  the IP packet is possibly forwarded to the ICMPAdaptor, or nuked, 
  depending on whether we want to generate an ICMP message or not.
  5. If the host is a router and the destination address is not ours,
  the next-hop IP is retrieved from the RoutingTable
  TTL (Time To Live) is subtracted by one
  if TTL=0, the packet is forwarded to the ICMPAdaptor
  
*/
class IpRoutingSession
  extends Session {

  /**
  */
  protected TcpipRoutingTable routingTable;
  
  /**
  */
  protected TcpipInterfaceTable interfaceTable;
  
  /**
  */
  public IpRoutingSession(State initialState, TcpipRoutingTable routingTable, TcpipInterfaceTable interfaceTable) {
  
  //+ IpRoutingSession::IpRoutingSession(State, TcpipRoutingTable, TcpipInterfaceTable)
      super(initialState);
      this.routingTable = routingTable;
      this.interfaceTable = interfaceTable;
  //- IpRoutingSession::IpRoutingSession(State, TcpipRoutingTable, TcpipInterfaceTable)
  }
  
} // IpRoutingSession

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
