/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IpRoutingState (implementation)
    Package: ip
    File:    IpRoutingState.java


*/
package fi.hut.tcm.jacob.tcpip.ip;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
    IpRoutingState
    
    This is the superclass of RoutingOnState and RoutingOffState, providing
    their common applyToNet() implementation and applyFromNet() interface.
*/
public abstract class IpRoutingState
  implements State {


  public abstract void applyFromNet(Message message, Session session,
				    Messenger messenger);

    /**
      Calls the Messenger's destinationLocal() method which tells if the
      destination IP address belongs to this machine. If the destination
      is local, then the message is sent back to the ReassemblySession.
      Otherwise, the messenger's setNextHopIp() method is called, and the
      packet is sent on  its way to the IpRoutingMux.
      
      */
  public void applyToNet(Message message, Session session, 
			 Messenger messenger) {
  //+ IpRoutingState::applyToNet(Message, Session, Messenger)
    IpRoutingToNetMessenger  msgr = (IpRoutingToNetMessenger) messenger;
    int dst = msgr.getDestIpAddr(message);
    IpRoutingSession sess = (IpRoutingSession) session;

    /*
     * Resolve the "next-hop" IP address, based on the ultimate destination IP
     */
    TcpipRoutingEntry entry = sess.routingTable.get(dst);
    if (entry == null) {
      System.err.println("Couldn't resolve nexthop address. " +
			 "Packet discarded.");
      ((IpRoutingSession)session).releaseMessage(message);
      return;
    }
    // nexthop is the default router if nothing more appropriate was found
    // and zero if the destination is directly reachable
    int nexthop = entry.getNexthop();
    if (nexthop == 0) {
      nexthop = dst;
    }
    msgr.setNextHopIpAddr(message, nexthop);
    msgr.setInterfaceId(message, entry.getInterfaceId());

    message.setMessenger(IpFragmentationToNetMessenger.instance);
  //- IpRoutingState::applyToNet(Message, Session, Messenger)
  }
    
}; //IpRoutingState

