/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  IcmpEchoAnswerAdaptor (implementation)
    Package: icmp
    File:    IcmpEchoAnswerAdaptor.java


*/
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import fi.hut.tcm.jacob.tcpip.icmp.*;

/**
    IcmpEchoAnswerAdaptor
    
    <p>Creates replies to incoming ICMP ECHO REQUEST messages.
*/
public class IcmpEchoAnswerAdaptor
  extends Adaptor {

    /**
        Reverses the source and destination addresses of the packet and
        changes the packet type to ICMP ECHO REPLY.
        
        @param message The <code>Message</code> carrying the incoming echo
        request
        @param message
    */
    public void adapt(Message message) {
    //+ IcmpEchoAnswerAdaptor::adapt(Message)
      //      byte tmp[] = new byte[6];
      int src;
      int dst;

      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      
      // Reverse source and destination addresses

      src = oob.getSrcIpAddr();
      dst = oob.getDestIpAddr();

      oob.setSrcIpAddr(dst);
      oob.setDestIpAddr(src);


      // Remember to set the correct messenger!
      message.setMessenger(IcmpChecksumToNetMessenger.instance);

      // The MUX's accessor handles changing the ICMP type
      // from 8 to 0.

      // The following seems necessary
      MessageTransporter mt =
      MessageTransporter.newMessageTransporter(sides[Side.A].getAttached(),
					       message);
      SimpleScheduler.instance.makeRunnable(mt);
      if (Icmp.DebugLevel > 2) {  // #Debug#
	System.out.println("Made new transporter in IcmpEchoAnswerAdaptor");
      }

    //- IcmpEchoAnswerAdaptor::adapt(Message)
    }

}; //IcmpEchoAnswerAdaptor

