/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**

    File:    IcmpEchoTest.java

    !!!!!!!!HANDMADE!!!!!!!

*/

import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import fi.hut.tcm.jacob.tcpip.icmp.*;

/**

  IcmpEchoTest

*/

public class IcmpEchoTest {
  
  // SimpleScheduler automatically starts travellerThread
  private final SimpleScheduler scheduler = SimpleScheduler.instance;

  // The framework doesn't currenly seem to tolerate
  // more than three multiple threads.
  private static final int NUMBER_APPLICATIONS = 5;
  
  public static void main(String[] argv) {

    if (argv.length != 0) {
      System.err.println("Usage: java IcmpEchoTest");
      System.exit(1);
    }

    IpAddress myAddr = new IpAddress("1.2.3.4");
    int myIpAddr = myAddr.toInt();

    IpAddress otherAddr = new IpAddress("10.0.10.20");
    int otherIpAddr = otherAddr.toInt();

    // Instances

    IcmpApplicationAdaptor masterApplAdap = 
      new IcmpApplicationAdaptor();
    Icmp icmpProtocol =
      new Icmp();
    IcmpBounceAdaptor bounAdap =
      new IcmpBounceAdaptor();
    
    // Testing cloning as well, not the protocol alone.

    //System.out.println("Original, unconnected protocol:");
    //icmpProtocol.debug();

    Icmp icmpProtocol2 = null;
    try {
      icmpProtocol2 = (Icmp)icmpProtocol.clone();
    } catch (CloneNotSupportedException e) {
      System.out.println("Cloning failed");
    }

    icmpProtocol = icmpProtocol2;

    //System.out.println("Newly cloned, unconnected protocol");
    //icmpProtocol.debug();

    Debugger.setDebuggable(icmpProtocol);

    try {
      masterApplAdap.getSide(Side.A).attach(icmpProtocol.getSide(Side.B));
      icmpProtocol.getSide(Side.B).attach(masterApplAdap.getSide(Side.A));
      bounAdap.getSide(Side.A).attach(icmpProtocol.getSide(Side.A));
      icmpProtocol.getSide(Side.A).attach(bounAdap.getSide(Side.A));
    } catch (SideException e) {
      System.out.println("Host #" + " failed to initialize");
      System.exit(2);
    }

    // Create NUMBER_APPLICATIONS applications to communicate
    // with the protocol.
    Application[] apps = new Application[NUMBER_APPLICATIONS];
    for(int j=0; j<NUMBER_APPLICATIONS; j++) {
      apps[j] = new Application(masterApplAdap,myIpAddr);
      apps[j].setRemoteIpAddress(otherIpAddr);
      apps[j].start();
    }

    // Wait here until all applications have finished.
    // Then exit.
    for(int j=0; j<NUMBER_APPLICATIONS; j++) {
      apps[j].waitTermination();
    }
    System.exit(0);

    /*
    IcmpApplicationAdaptor workerApplAdap =
      masterApplAdap.connect();
    if (workerApplAdap == null ) {
      System.err.println("Unable to connect to Icmp Protocol");
      System.exit(1);
    }

    //System.out.println("Newly cloned, now connected protocol with 1 app session");
    //icmpProtocol.debug();

    int seq;
    for (int i = 1; i < 2; i++) {  // ping 5 times, for now
      seq = i;
      System.out.println("Sending ICMP Echo, seq = " + seq);
      workerApplAdap.send(otherIpAddr, seq);
      Thread.currentThread().yield();
      try {
	Thread.sleep(1000);
      } catch (InterruptedException e){ 
	// do nothing.
      }
    }
    */

  }
}
