/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**

    File:    Test.java

    For testing the UDP protocol.

*/

import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import fi.hut.tcm.jacob.tcpip.udp.*;

/**

  Test

  For testing the UDP protocol.

*/

public class Test {
  
  // SimpleScheduler automatically starts travellerThread
  private final SimpleScheduler scheduler = SimpleScheduler.instance;

  // The framework doesn't currenly seem to tolerate
  // more than three multiple threads.
  private static final int NUMBER_APPLICATIONS = 5;

  private static final int FIRST_APP_PORT = 4000;
  
  public static void main(String[] argv) {

    if (argv.length != 0) {
      System.err.println("Usage: java Test");
      System.exit(1);
    }

    // We do not have multiple IP addresses, or even if we do,
    // we still always use this one.
    // We will, however, let each application separately specify
    // that this is the address they want to use, so that
    // the use of multiple addresses will be possible if necessary.
    IpAddress myAddr = new IpAddress("1.2.3.4");
    int myIpAddr = myAddr.toInt();

    // We only wish to communicate with one machine,
    // and this is its address.
    IpAddress otherAddr = new IpAddress("10.0.10.20");
    int otherIpAddr = otherAddr.toInt();

    // Instances

    UdpApplicationAdaptor masterApplAdap = new UdpApplicationAdaptor();
    UDP udpProtocol = new UDP();
    // Swaps the source and destination addresses and port numbers,
    // and gives the packet back to the protocol.
    NetAdaptor netAdaptor = new NetAdaptor();

    // Testing cloning as well, not the protocol alone.

    //System.out.println("Original, unconnected protocol:");
    //icmpProtocol.debug();

    /* let's not try this yet
    UDP udpProtocol2 = null;
    try {
      udpProtocol2 = (UDP)udpProtocol.clone();
    } catch (CloneNotSupportedException e) {
      System.out.println("Cloning failed");
    }
    udpProtocol = udpProtocol2;
    */

    //System.out.println("Newly cloned, unconnected protocol");
    //icmpProtocol.debug();

    //Debugger.setDebuggable(icmpProtocol);

    // Connect the protocol into the outside world.
    try {
      masterApplAdap.getSide(Side.A).attach(udpProtocol.getSide(Side.B));
      udpProtocol.getSide(Side.B).attach(masterApplAdap.getSide(Side.A));
      netAdaptor.getSide(Side.A).attach(udpProtocol.getSide(Side.A));
      udpProtocol.getSide(Side.A).attach(netAdaptor.getSide(Side.A));
    } catch (SideException e) {
      System.out.println("Host #" + " failed to initialize");
      System.exit(2);
    }

    // Create NUMBER_APPLICATIONS applications to communicate
    // with the protocol.
    Application[] apps = new Application[NUMBER_APPLICATIONS];
    for(int j=0; j<NUMBER_APPLICATIONS; j++) {
      // The application probably knows the IP address of
      // the machine it is running on, but it cannot know for
      // sure if the port it wants is free at the time
      // it asks the protocol for a new session.
      apps[j] = new Application(masterApplAdap,myIpAddr,FIRST_APP_PORT+j);
      apps[j].setDestination(otherIpAddr,FIRST_APP_PORT+j);
      apps[j].start();
    }

    // Wait here until all applications have finished.
    // Then exit.
    for(int j=0; j<NUMBER_APPLICATIONS; j++) {
      apps[j].waitTermination();
    }
    System.exit(0);

  }
}
