/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
    Copyright (C) 1997 Pekka Nikander & Juha Prssinen.
    All rights Reserved.

    XXX   %Z%%M% version %I% date:%G%

    Module:  UdpApplicationAdaptor (implementation)
    Package: Udp
    File:    UdpApplicationAdaptor.java


*/

import fi.hut.tcm.jacob.conduits.SimpleScheduler;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import fi.hut.tcm.jacob.tcpip.udp.*;

/**
  UdpApplicationAdaptor
  
  <p>Creates UDP ECHO REQUEST messages and receives echo replies.
  */
public class UdpApplicationAdaptor
extends Adaptor {



  private MessageAction messageAction;

  public void setMessageAction(MessageAction action)
    {
      messageAction = action;
    }


  
  /**
    Returns a reference to the new, connected adaptor iff
    successful.
    */
  public UdpApplicationAdaptor connect(int port) {
    // We'll have to send an Installer to the UDP protocol,
    // requesting service.
    
    UdpApplicationAdaptor workerAdaptor = new UdpApplicationAdaptor();
    
    Side[] oobsides = new Side[2];
    oobsides[Side.A] = null;   // Factory/mux fills in this side
    oobsides[Side.B] = workerAdaptor.getSide(Side.A);
    Object[] oobkeys = new Object[2];
    oobkeys[Side.A] = new Integer(port);
    oobkeys[Side.B] = null; // not needed
    InstallOob ioob = new InstallOob(oobsides, oobkeys, null); // no prototype
    // The message content is ours to choose, so we include
    // a reference to the workerAdaptor so that we know for
    // which adaptor the session was created when we receive
    // installation complete notification.
    // We can then in turn inform that adaptor about the results
    // of the operation.
    Message msg =
      Message.newMessage((Object)workerAdaptor, Installer.instance, ioob);
    if(sides[Side.A] != null) {
      MessageTransporter mt =
	MessageTransporter.newMessageTransporter(this.sides[Side.A].
						 getAttached(), 
						 msg);
      SimpleScheduler.instance.makeRunnable(mt);
      Thread.currentThread().yield();
    }
    // Wait for a reply before returning "OK" to the caller.
    if(workerAdaptor.waitInstall())
      return workerAdaptor;
    return null;
  }
  
  /**
    Returns true iff the operation is successful.
    */
  public boolean disconnect(UdpApplicationAdaptor worker)
  {
    // We'll have to send an Uninstaller to the UDP protocol,
    // requesting service.
    //
    UninstallOob uioob = new UninstallOob((Conduit)worker);
    Message msg = Message.newMessage(worker, Uninstaller.instance, uioob);
    if(sides[Side.A] != null) {
      MessageTransporter mt =
	MessageTransporter.newMessageTransporter(this.sides[Side.A].
						 getAttached(),msg);
      SimpleScheduler.instance.makeRunnable(mt);
      Thread.currentThread().yield();
    }
    // Wait for a reply before returning.
    return worker.waitUninstall();
  }
  


  
  /**
     Receives replies to echo requests.
     
     @param message The <code>Message</code> carrying the echo reply
  */
  public void adapt(Message message) {
    //+ UdpApplicationAdaptor::adapt(Message)

      if(Debugger.level > 3)
	System.out.println("Now in UdpApplicationAdaptor.adapt()");

      // Incoming replies to packets we have sent should arrive here.

      if(filter(message)) return;

      Object buf = message.getContents();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      int len = UtilMsgr.i.getLength(buf);
      
      int src = oob.getSrcIpAddr();
      int dst = oob.getDestIpAddr();

      String txt = "Out of band: src = " + IpAddress.toString(src) +
	", dst = " + IpAddress.toString(dst);

      // The contents of the actual message are not checked here,
      // but we will let it be checked by the application if it wants
      // to do so.
      if(messageAction != null)
	messageAction.act(buf);

    //- UdpApplicationAdaptor::adapt(Message)
    }

    /**
        Send an UDP ECHO REQUEST message.
        
        @param    srcIp    source IP address
	@param    srcPort  source port
        @param    dstIp    destination IP address
	@param    dstPort  destination port
        @param    ident    packet identifier
        @param    seq      packet sequence number
    */
  public void send(int srcIp, int srcPort,
		   int dstIp, int dstPort,
		   int ident, int seq)
    {
      //+ UdpApplicationAdaptor::send(int, int)

      // Maximum size for IP header is 60 octets. (RFC 791).
      // Allow for that, plus some extra header room for ethernet
      // and unforseen requirements. = total of 96 octets.
      
      // (MAXSIZE, INITIALOFFSET, INITIALLENGTH);
      Object buf = BufferMessenger.newBuffer(256, 96, 32);
      
      // Payload.
      // (We can put any data that the application might
      // need into the payload, the protocol shouldn't care.
      // The header will be prepended in UdpHeaderState.)
      UtilMsgr.i.setShort(buf,0,ident);
      UtilMsgr.i.setShort(buf,1,seq);
      UtilMsgr.i.setShort(buf,2,srcPort);
      UtilMsgr.i.setShort(buf,3,dstPort);
      UtilMsgr.i.setInt(buf,2,srcIp);
      UtilMsgr.i.setInt(buf,3,dstIp);
      
      TcpipOutOfBand oob = new TcpipOutOfBand();
      oob.setDestIpAddr(dstIp);
      oob.setSrcIpAddr(srcIp);
      oob.setDestPort(dstPort);
      oob.setSrcPort(srcPort);
      
      Message msg =
	Message.newMessage(buf, UdpHeaderToNetMessenger.instance, oob);
      
      if (sides[Side.A] != null) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sides[Side.A].getAttached(),
						   msg);
	SimpleScheduler.instance.makeRunnable(mt);
      }
      //- UdpApplicationAdaptor::send(int, int)
    }
  











  private boolean doWaitInstallation = true;
  private boolean installationOkay = false;
    
    /**
      Wait until installation notification has been received,
      returning true if installation was successful.
      */
  public synchronized boolean waitInstall()
  {
    while(doWaitInstallation) {
      try {
	wait();
      } catch(InterruptedException e) {
	System.out.println("wait() interrupted");
      }
    }
    doWaitInstallation = true;
    return installationOkay;
  }

  public synchronized void installComplete(boolean okay)
  {
    installationOkay = okay;
    doWaitInstallation = false;
    notifyAll();
  }


  private boolean doWaitUninstallation = true;
  private boolean uninstallationOkay = false;

    /**
      Wait until uninstallation notification has been received,
      returning true if uninstallation was successful.
      */
  public synchronized boolean waitUninstall()
  {
    while(doWaitUninstallation) {
      try {
	wait();
      } catch(InterruptedException e) {
	System.out.println("wait() interrupted");
      }
    }
    doWaitUninstallation = true;
    return uninstallationOkay;
  }

  public synchronized void uninstallComplete(boolean okay)
  {
    uninstallationOkay = okay;
    doWaitUninstallation = false;
    notifyAll();
  }

    
    /**
      Checks to see if the message passed in is a notification message.
      If so, checks what it is notification of.
      */
  protected boolean filter(Message message)
  {
    //+ UdpApplicationAdaptor::filter(Message)
    Messenger messenger = message.getMessenger();
    if(messenger instanceof NotifyMessenger) {
      Message request = ((NotifyMessenger)messenger).getRequest(message);
      Messenger reqMsgr = request.getMessenger();
      if(reqMsgr instanceof InstallMessenger) {
	UdpApplicationAdaptor worker =
	  (UdpApplicationAdaptor)request.getContents();
	if(worker == null)
	  System.out.println("Assertion failed in UdpApplicationAdaptor." +
			     "filter()");
	if(((NotifyOob)message.getOob()).getStatusCode()
	   ==NotifyMessenger.SUCCESS)
	  worker.installComplete(true);
	else
	  worker.installComplete(false);
	if(Debugger.level > 1)
	  System.out.println("Got installation notification, status " +
			     ((NotifyOob)message.getOob()).getStatusCode());
      } else if(reqMsgr instanceof UninstallMessenger) {
	UdpApplicationAdaptor worker =
	  (UdpApplicationAdaptor)request.getContents();
	if(worker == null)
	  System.out.println("Assertion failed in UdpApplicationAdaptor." +
			     "filter()");
	if(((NotifyOob)message.getOob()).getStatusCode()
	   ==NotifyMessenger.SUCCESS)
	  worker.uninstallComplete(true);
	else
	  worker.uninstallComplete(false);
	if(Debugger.level > 1)
	  System.out.println("Got uninstallation notification, status " +
			     ((NotifyOob)message.getOob()).getStatusCode());
      }
      return true;
    }
    return false;
    //- UdpApplicationAdaptor::filter(Message)
  }


}; //UdpApplicationAdaptor

