/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  XXX   %Z%%M% version %I% date:%G%
  Component:  UDP
  Package: udp
  File:    UDP.java
*/

package fi.hut.tcm.jacob.tcpip.udp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
  UDP
  
      UDP
  
      <p>
      User Datagram Protocol.
  
*/
public class UDP
  extends Protocol {

  /**
    This value is one of the members of the UDP pseudo header,
    which is used during checksum calculation.
    Protocol field in IP header.
    
  */
  public  int UDP_PROTOL_ID = 17;
  
  /**
  */
  public UDP() {
  //+ UDP::UDP( )
    super();

    // Mux, and its accessor and terminator.
    UdpPortAccessor portAccessor = new UdpPortAccessor();
    Mux portMux = new Mux(portAccessor);
    Terminator portMuxTerminator = new Terminator(); 

    // Factory, its prototype, and terminator.
    UdpHeaderSession headerSessionProto = 
      new UdpHeaderSession(UdpHeaderState.instance);
    Side[] sessionSides = new Side[2];
    sessionSides[Side.A] = portMux.getSide(Side.B);
    sessionSides[Side.B] = null;  // The installer will know this side.
    ConduitFactory sessionFactory =
      new ConduitFactory(headerSessionProto, sessionSides);
    Terminator factoryTerminator = new Terminator();
    
    // Checksum session.
    UdpChecksumSession sumSession = 
      new UdpChecksumSession(UdpChecksumState.instance);

    // Now connect the created components.
    try {
      sides[INSIDE_B].attach(sessionFactory.getSide(Side.B));
      sessionFactory.getSide(Side.B).attach(sides[INSIDE_B]);

      sessionFactory.getSide(Side.A).attach(factoryTerminator.getSide(Side.A));
      factoryTerminator.getSide(Side.A).attach(sessionFactory.getSide(Side.A));

      sides[INSIDE_A].attach(sumSession.getSide(Side.A));
      sumSession.getSide(Side.A).attach(sides[INSIDE_A]);

      sumSession.getSide(Side.B).attach(portMux.getSide(Side.A));
      portMux.getSide(Side.A).attach(sumSession.getSide(Side.B));

      portMux.getSide(Side.B).attach(portMuxTerminator.getSide(Side.A));
      portMuxTerminator.getSide(Side.A).attach(portMux.getSide(Side.B));
    } catch (SideException e) {
      System.out.println("UDP failed to initialize: " + e);
      System.exit(2);
    }
    
    // Set a sensible default messenger, as it doesn't make sense
    // that objects external to the protocol would have to know
    // about the internals of the protocol, i.e. which
    // conduit will be encountered first and what kind of messenger
    // it needs.
    (sides[Side.A]).setMessenger(UdpChecksumFromNetMessenger.instance);
    
  //- UDP::UDP( )
  }
  
  /**
  */
  public Object clone()
    throws CloneNotSupportedException  {
  //+ UDP::clone( )
    Conduit c;
    Protocol clone = (Protocol)super.clone();
    // Get a reference to the ConduitFactory.
    c = getConduitCheckConnection(clone.getSide(INSIDE_B));
    if(!(c instanceof ConduitFactory))
      System.out.println("Assertion failed in UDP.clone()");
    ConduitFactory sessionFactory = (ConduitFactory)c;
    // Get a reference to the port mux.
    c = getConduitCheckConnection(clone.getSide(INSIDE_A));
    c = getConduitCheckConnection(c.getSide(Side.B));
    c = getConduitCheckConnection(c.getSide(2));
    if(!(c instanceof Mux))
      System.out.println("Assertion failed in UDP.clone()");
    Mux portMux = (Mux)c;
    sessionFactory.setEndpoint(Side.A,portMux.getSide(Side.B));
    // The installer will know the B side of the session to be created.
    sessionFactory.setEndpoint(Side.B,null);
    return (Object)clone;
  //- UDP::clone( )
  }
  
  /**
  */
  private Conduit getConduitCheckConnection(Side side)
    throws CloneNotSupportedException  {
  //+ UDP::getConduitCheckConnection(Side)
    if(side == null)
      System.out.println("check connection failed (" + 1 + ")");
    Side side2 = side.getAttached();
    if(side2 == null)
      System.out.println("check connection failed (" + 2 + ")");
    if(side2.getAttached() != side)
      System.out.println("check connection failed (" + 3 + ")");
    Conduit conduit = side2.getConduit();
    if(conduit == null)
      System.out.println("check connection failed (" + 4 + ")");
    return conduit;
  //- UDP::getConduitCheckConnection(Side)
  }
  
  /**
  */
  public void bindToPort() {
  //+ UDP::bindToPort( )
  //- UDP::bindToPort( )
  }
  
  /**
  */
  public void releasePort() {
  //+ UDP::releasePort( )
  //- UDP::releasePort( )
  }
  
  /**
  */
  public void handleICMPMessage() {
  //+ UDP::handleICMPMessage( )
  //- UDP::handleICMPMessage( )
  }
  
  /**
  */
  public void deepcopy() {
  //+ UDP::deepcopy( )
  //- UDP::deepcopy( )
  }
  
} // UDP

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
