/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  XXX   %Z%%M% version %I% date:%G%
  Component:  UdpAdaptor
  Package: udp
  File:    UdpAdaptor.java
*/

package fi.hut.tcm.jacob.tcpip.udp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;
import java.net.*;

/**
  UdpAdaptor
  
  
*/
public class UdpAdaptor
  extends Adaptor {

  /**
  */
  private JacobDatagramSocket socket;
  
  /**
  */
  private  boolean uninstallationOkay = false;
  
  /**
  */
  private  boolean installationOkay = false;
  
  /**
  */
  private  boolean doWaitUninstallation = true;
  
  /**
  */
  private  boolean doWaitInstallation = true;
  
  /**
  */
  public void setMessageAction(JacobDatagramSocket socket) {
  //+ UdpAdaptor::setMessageAction(JacobDatagramSocket)
      this.socket = socket;
  //- UdpAdaptor::setMessageAction(JacobDatagramSocket)
  }
  
  /**
  */
  public void send(int srcIp, int srcPort, DatagramPacket p) {
  //+ UdpAdaptor::send(int, int, DatagramPacket)

      // Maximum size for IP header is 60 octets. (RFC 791).
      // Allow for that, plus some extra header room for ethernet
      // and unforseen requirements. = total of 96 octets.
      
      int length = p.getLength();

      // (MAXSIZE, INITIALOFFSET, INITIALLENGTH);
      Object buf = BufferMessenger.newBuffer(96 + length, 96, p.getData());
      
      byte[] destIpAddr = p.getAddress().getAddress();
      int dstIp =
	((int) destIpAddr[0] & 0xff) << 24 |      
	((int) destIpAddr[1] & 0xff) << 16 |      
	((int) destIpAddr[2] & 0xff) <<  8 |      
	((int) destIpAddr[3] & 0xff) <<  0;

      TcpipOutOfBand oob = new TcpipOutOfBand();
      oob.setDestIpAddr(dstIp);
      oob.setSrcIpAddr(srcIp);
      oob.setDestPort(p.getPort());
      oob.setSrcPort(srcPort);
      
      Message msg =
	Message.newMessage(buf, UdpHeaderToNetMessenger.instance, oob);
      
      if (sides[Side.A] != null) {
	MessageTransporter mt =
	  MessageTransporter.newMessageTransporter(sides[Side.A].getAttached(),
						   msg);
	SimpleScheduler.instance.makeRunnable(mt);
      }
  //- UdpAdaptor::send(int, int, DatagramPacket)
  }
  
  /**
  */
  public synchronized void uninstallComplete(boolean okay) {
  //+ UdpAdaptor::uninstallComplete(boolean)
    uninstallationOkay = okay;
    doWaitUninstallation = false;
    notifyAll();
  //- UdpAdaptor::uninstallComplete(boolean)
  }
  
  /**
  */
  public synchronized void installComplete(boolean okay) {
  //+ UdpAdaptor::installComplete(boolean)
    installationOkay = okay;
    doWaitInstallation = false;
    notifyAll();
  //- UdpAdaptor::installComplete(boolean)
  }
  
  /**
  */
  public synchronized boolean waitUninstall() {
  //+ UdpAdaptor::waitUninstall( )
    while(doWaitUninstallation) {
      try {
	wait();
      } catch(InterruptedException e) {
	System.out.println("wait() interrupted");
      }
    }
    doWaitUninstallation = true;
    return uninstallationOkay;
  //- UdpAdaptor::waitUninstall( )
  }
  
  /**
  */
  public synchronized boolean waitInstall() {
  //+ UdpAdaptor::waitInstall( )
    while(doWaitInstallation) {
      try {
	wait();
      } catch(InterruptedException e) {
	System.out.println("wait() interrupted");
      }
    }
    doWaitInstallation = true;
    return installationOkay;
  //- UdpAdaptor::waitInstall( )
  }
  
  /**
  */
  protected boolean filter(Message message) {
  //+ UdpAdaptor::filter(Message)
    Messenger messenger = message.getMessenger();
    if(messenger instanceof NotifyMessenger) {
      Message request = ((NotifyMessenger)messenger).getRequest(message);
      Messenger reqMsgr = request.getMessenger();
      if(reqMsgr instanceof InstallMessenger) {
	UdpAdaptor worker =
	  (UdpAdaptor)request.getContents();
	if(worker == null)
	  System.out.println("Assertion failed in UdpAdaptor." +
			     "filter()");
	if(((NotifyOob)message.getOob()).getStatusCode()
	   ==NotifyMessenger.SUCCESS)
	  worker.installComplete(true);
	else
	  worker.installComplete(false);
	if(Debugger.level > 1)
	  System.out.println("Got installation notification, status " +
			     ((NotifyOob)message.getOob()).getStatusCode());
      } else if(reqMsgr instanceof UninstallMessenger) {
	UdpAdaptor worker =
	  (UdpAdaptor)request.getContents();
	if(worker == null)
	  System.out.println("Assertion failed in UdpAdaptor." +
			     "filter()");
	if(((NotifyOob)message.getOob()).getStatusCode()
	   ==NotifyMessenger.SUCCESS)
	  worker.uninstallComplete(true);
	else
	  worker.uninstallComplete(false);
	if(Debugger.level > 1)
	  System.out.println("Got uninstallation notification, status " +
			     ((NotifyOob)message.getOob()).getStatusCode());
      }
      return true;
    }
    return false;
  //- UdpAdaptor::filter(Message)
  }
  
  /**
  */
  public void adapt(Message message) {
  //+ UdpAdaptor::adapt(Message)

      if(Debugger.level > 3)
	System.out.println("Now in UdpAdaptor.adapt()");

      // Incoming replies to packets we have sent should arrive here.

      if(filter(message)) return;
      //////////
      Object buf = message.getContents();
      TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
      int len = UtilMsgr.i.getLength(buf);

      byte[] data = UtilMsgr.i.extractData(buf);

      int srcIpAddr = oob.getSrcIpAddr();
      String srcIpStr = new String(((srcIpAddr >> 24) & 0xff) + "." + 
				   ((srcIpAddr >> 16) & 0xff) + "." + 
				   ((srcIpAddr >>  8) & 0xff) + "." + 
				   ((srcIpAddr >>  0) & 0xff)); 
      try {
	if (socket != null) {
	  DatagramPacket packet = 
	    new DatagramPacket(data, 
			       data.length, 
			       InetAddress.getByName(srcIpStr),
			       oob.getSrcPort());
	  socket.put(packet);
	} else {
	  System.out.println("UdpAdaptor: TROWING AWAY PACKET!");
	}
      } catch (UnknownHostException e) {
	System.err.println("Host " + srcIpStr + 
			   " caused an UnknownHostException");
      } 

  //- UdpAdaptor::adapt(Message)
  }
  
  /**
  */
  public boolean disconnect(UdpAdaptor worker) {
  //+ UdpAdaptor::disconnect(UdpAdaptor)
    // We'll have to send an Uninstaller to the UDP protocol,
    // requesting service.
    //
    UninstallOob uioob = new UninstallOob((Conduit)worker);
    Message msg = Message.newMessage(worker, Uninstaller.instance, uioob);
    if(sides[Side.A] != null) {
      MessageTransporter mt =
	MessageTransporter.newMessageTransporter(this.sides[Side.A].
						 getAttached(),msg);
      SimpleScheduler.instance.makeRunnable(mt);
      Thread.currentThread().yield();
    }
    // Wait for a reply before returning.
    return worker.waitUninstall();
  //- UdpAdaptor::disconnect(UdpAdaptor)
  }
  
  /**
  */
  public UdpAdaptor connect(int port) {
  //+ UdpAdaptor::connect(int)
    // We'll have to send an Installer to the UDP protocol,
    // requesting service.
    
    UdpAdaptor workerAdaptor = new UdpAdaptor();
    
    Side[] oobsides = new Side[2];
    oobsides[Side.A] = null;   // Factory/mux fills in this side
    oobsides[Side.B] = workerAdaptor.getSide(Side.A);
    Object[] oobkeys = new Object[2];
    oobkeys[Side.A] = new Integer(port);
    oobkeys[Side.B] = null; // not needed
    InstallOob ioob = new InstallOob(oobsides, oobkeys, null); // no prototype
    // The message content is ours to choose, so we include
    // a reference to the workerAdaptor so that we know for
    // which adaptor the session was created when we receive
    // installation complete notification.
    // We can then in turn inform that adaptor about the results
    // of the operation.
    Message msg =
      Message.newMessage((Object)workerAdaptor, Installer.instance, ioob);
    if(sides[Side.A] != null) {
      MessageTransporter mt =
	MessageTransporter.newMessageTransporter(this.sides[Side.A].
						 getAttached(), 
						 msg);
      SimpleScheduler.instance.makeRunnable(mt);
      Thread.currentThread().yield();
    }
    // Wait for a reply before returning "OK" to the caller.
    if(workerAdaptor.waitInstall())
      return workerAdaptor;
    return null;
  //- UdpAdaptor::connect(int)
  }
  
} // UdpAdaptor

// +*************** UNUSED CODE ***************

// -*************** UNUSED CODE ***************
