/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  XXX   %Z%%M% version %I% date:%G%
  Component:  UdpChecksumFromNetMessenger
  Package: udp
  File:    UdpChecksumFromNetMessenger.java
*/

package fi.hut.tcm.jacob.tcpip.udp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
  UdpChecksumFromNetMessenger
  
  
*/
public class UdpChecksumFromNetMessenger
  extends UdpMessenger {

  /**
  */
  public static UdpChecksumFromNetMessenger instance = new UdpChecksumFromNetMessenger();
  
  /**
  */
  public void apply(Message message, State state, Session session) {
  //+ UdpChecksumFromNetMessenger::apply(Message, State, Session)
    UdpChecksumState udpState = (UdpChecksumState) state; 
    udpState.applyFromNet(message, this, session);
  //- UdpChecksumFromNetMessenger::apply(Message, State, Session)
  }
  
  /**
  */
  private UdpChecksumFromNetMessenger() {
  //+ UdpChecksumFromNetMessenger::UdpChecksumFromNetMessenger( )
  //- UdpChecksumFromNetMessenger::UdpChecksumFromNetMessenger( )
  }
  
  /**
  */
  public boolean validChecksum(Message message) {
  //+ UdpChecksumFromNetMessenger::validChecksum(Message)

    Object buf = message.getContents();
    int len = getUdpLength(message);

    TcpipOutOfBand oob = (TcpipOutOfBand) message.getOob();
    int srcIpAddr = oob.getSrcIpAddr();
    int destIpAddr = oob.getDestIpAddr();
    int csum = 0;

    // difference from UdpChecksumToNetMessenger: save the value first!
    int saved_checksum = getChecksum(message);
    if (saved_checksum == 0) {  // 0 = there is no checksum
      return true;
    }

    // calculations needs to have existing checksum field contains zeros:
    setChecksum(message, 0);

    // Calculate from a "pseudo header"
    csum += ((srcIpAddr >> 16) & 0xffff) + (srcIpAddr & 0xffff);
    csum += ((destIpAddr >> 16) & 0xffff) + (destIpAddr & 0xffff);
    csum += 17;  // zero field + UDP protocol id
    csum += len; // UDP length
      
    // Calculate from the datagram
    for (int i = 0; i < (len / 2) * 2; i += 2) {
      csum += getShortUnaligned(buf,i);
    }
    if ( (len % 2) == 1 ) {  // we had an odd size packet
      csum += (getByte(buf,len-1) << 8);
    }
    
    // 32-bits to 16-bit folding and 1's complement
    while ((csum >> 16) > 0) {
      csum = (csum & 0xffff) + ((csum >> 16) & 0xffff);
    }
    csum = ~csum & 0xffff;

    if (saved_checksum == csum) {
      // DEBUG
      //System.out.println("UdpChecksumFromNetMessenger: checksum OK!");
      return true;
    } else {
      System.out.println("UdpChecksumFromNetMessenger: checksum FAILED!!!");
      System.out.println("Packet's checksum was " + saved_checksum +
			 " calculated checksum was " + csum);
      System.out.println("Source Ip: " + IpAddress.toString(srcIpAddr));
      System.out.println("Destination Ip: " + IpAddress.toString(destIpAddr));
      System.out.println("Packet length: " + len);
      return false;
    }

  //- UdpChecksumFromNetMessenger::validChecksum(Message)
  }
  
} // UdpChecksumFromNetMessenger

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
