/*
  Copyright (c) 1998 Helsinki University of Technology.
  All rights reserved.

  Redistribution and use in source and binary forms are permitted
  provided that the above copyright notice and this paragraph are
  duplicated in all such forms and that any documentation,
  advertising materials, and other materials related to such
  distribution and use acknowledge that the software was developed
  by Helsinki University of Technology, Espoo.  The name of the
  University may not be used to endorse or promote products derived
  from this software without specific prior written permission.
  THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
  WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
 */

/**
  XXX   %Z%%M% version %I% date:%G%
  Component:  UdpChecksumToNetMessenger
  Package: udp
  File:    UdpChecksumToNetMessenger.java
*/

package fi.hut.tcm.jacob.tcpip.udp;
import fi.hut.tcm.jacob.conduits.*;
import fi.hut.tcm.jacob.tcpip.*;

/**
  UdpChecksumToNetMessenger
  
  
*/
public class UdpChecksumToNetMessenger
  extends UdpMessenger {

  /**
  */
  public static UdpChecksumToNetMessenger instance = new UdpChecksumToNetMessenger();
  
  /**
  */
  public void apply(Message message, State state, Session session) {
  //+ UdpChecksumToNetMessenger::apply(Message, State, Session)
    UdpChecksumState udpState = (UdpChecksumState) state; 
    udpState.applyToNet(message, session, this);
  //- UdpChecksumToNetMessenger::apply(Message, State, Session)
  }
  
  /**
  */
  public void calculateChecksum(Message message, int srcIpAddr, int destIpAddr) {
  //+ UdpChecksumToNetMessenger::calculateChecksum(Message, int, int)
    
    Object buf =  message.getContents();

    int len = getUdpLength(message);
    
    int csum = 0;
    
    // make sure the existing checksum field contains zeros:
    setChecksum(message, 0);
      
    // Calculate from a "pseudo header"
    csum += ((srcIpAddr >> 16) & 0xffff) + (srcIpAddr & 0xffff);
    csum += ((destIpAddr >> 16) & 0xffff) + (destIpAddr & 0xffff);
    csum += 17;  // zero field + UDP protocol id
    csum += len; // UDP length

    // Calculate from the datagram
    for (int i = 0; i < (len / 2) * 2; i += 2) {
      csum += getShortUnaligned(buf,i);
    }
    if ( (len % 2) == 1 ) {  // we had an odd size packet
      csum += (getByte(buf,len-1) << 8);
    }

    // 32-bits to 16-bit folding and 1's complement
    while ((csum >> 16) > 0) {
      csum = (csum & 0xffff) + ((csum >> 16) & 0xffff);
    }
    csum = ~csum & 0xffff;

    if (csum == 0) {  // 0 is special, it means no checksum.
      csum = 0xffff;  // this is specified in RFC768.
    }
    setChecksum(message, csum);

  //- UdpChecksumToNetMessenger::calculateChecksum(Message, int, int)
  }
  
  /**
  */
  private UdpChecksumToNetMessenger() {
  //+ UdpChecksumToNetMessenger::UdpChecksumToNetMessenger( )
  //- UdpChecksumToNetMessenger::UdpChecksumToNetMessenger( )
  }
  
} // UdpChecksumToNetMessenger

// +*************** UNUSED CODE ***************
// -*************** UNUSED CODE ***************
